@@ -114,27 +114,40 @@ a forward reference to a class in a top-level annotation. Example:
 Note that a forward reference in a function signature won't trigger another
 pass, since all functions are processed only after the top level has been fully
 analyzed.
-
-You can use `api.options.new_semantic_analyzer` to check whether the new
-semantic analyzer is enabled (it's always true in mypy 0.730 and later).
 """
 
-from abc import abstractmethod
-from typing import Any, Callable, List, Tuple, Optional, NamedTuple, TypeVar, Dict, Union
-from mypy_extensions import trait, mypyc_attr
+from __future__ import annotations
 
+from abc import abstractmethod
+from typing import Any, Callable, NamedTuple, TypeVar
+
+from mypy_extensions import mypyc_attr, trait
+
+from mypy.errorcodes import ErrorCode
+from mypy.lookup import lookup_fully_qualified
+from mypy.message_registry import ErrorMessage
+from mypy.messages import MessageBuilder
 from mypy.nodes import (
-    Expression, Context, ClassDef, SymbolTableNode, MypyFile, CallExpr, ArgKind, TypeInfo
+    ArgKind,
+    CallExpr,
+    ClassDef,
+    Context,
+    Expression,
+    MypyFile,
+    SymbolTableNode,
+    TypeInfo,
 )
+from mypy.options import Options
 from mypy.tvar_scope import TypeVarLikeScope
 from mypy.types import (
-    Type, Instance, CallableType, TypeList, UnboundType, ProperType, FunctionLike
+    CallableType,
+    FunctionLike,
+    Instance,
+    ProperType,
+    Type,
+    TypeList,
+    UnboundType,
 )
-from mypy.messages import MessageBuilder
-from mypy.options import Options
-from mypy.lookup import lookup_fully_qualified
-from mypy.errorcodes import ErrorCode
-from mypy.message_registry import ErrorMessage
 
 
 @trait
@@ -152,24 +165,24 @@ class TypeAnalyzerPluginInterface:
     options: Options
 
     @abstractmethod
-    def fail(self, msg: str, ctx: Context, *, code: Optional[ErrorCode] = None) -> None:
+    def fail(self, msg: str, ctx: Context, *, code: ErrorCode | None = None) -> None:
         """Emit an error message at given location."""
         raise NotImplementedError
 
     @abstractmethod
-    def named_type(self, name: str, args: List[Type]) -> Instance:
+    def named_type(self, fullname: str, args: list[Type], /) -> Instance:
         """Construct an instance of a builtin type with given name."""
         raise NotImplementedError
 
     @abstractmethod
-    def analyze_type(self, typ: Type) -> Type:
+    def analyze_type(self, typ: Type, /) -> Type:
         """Analyze an unbound type using the default mypy logic."""
         raise NotImplementedError
 
     @abstractmethod
-    def analyze_callable_args(self, arglist: TypeList) -> Optional[Tuple[List[Type],
-                                                                         List[ArgKind],
-                                                                         List[Optional[str]]]]:
+    def analyze_callable_args(
+        self, arglist: TypeList
+    ) -> tuple[list[Type], list[ArgKind], list[str | None]] | None:
         """Find types, kinds, and names of arguments from extended callable syntax."""
         raise NotImplementedError
 
@@ -177,7 +190,7 @@ class TypeAnalyzerPluginInterface:
 # A context for a hook that semantically analyzes an unbound type.
 class AnalyzeTypeContext(NamedTuple):
     type: UnboundType  # Type to analyze
-    context: Context   # Relevant location context (e.g. for error messages)
+    context: Context  # Relevant location context (e.g. for error messages)
     api: TypeAnalyzerPluginInterface
 
 
@@ -194,7 +207,7 @@ class CommonPluginApi:
     options: Options
 
     @abstractmethod
-    def lookup_fully_qualified(self, fullname: str) -> Optional[SymbolTableNode]:
+    def lookup_fully_qualified(self, fullname: str) -> SymbolTableNode | None:
         """Lookup a symbol by its full name (including module).
 
         This lookup function available for all plugins. Return None if a name
@@ -218,19 +231,25 @@ class CheckerPluginInterface:
     # Type context for type inference
     @property
     @abstractmethod
-    def type_context(self) -> List[Optional[Type]]:
+    def type_context(self) -> list[Type | None]:
         """Return the type context of the plugin"""
         raise NotImplementedError
 
     @abstractmethod
-    def fail(self, msg: Union[str, ErrorMessage], ctx: Context, *,
-             code: Optional[ErrorCode] = None) -> None:
+    def fail(
+        self, msg: str | ErrorMessage, ctx: Context, /, *, code: ErrorCode | None = None
+    ) -> None:
         """Emit an error message at given location."""
         raise NotImplementedError
 
     @abstractmethod
-    def named_generic_type(self, name: str, args: List[Type]) -> Instance:
-        """Construct an instance of a builtin type with given type arguments."""
+    def named_generic_type(self, name: str, args: list[Type]) -> Instance:
+        """Construct an instance of a generic type with given type arguments."""
+        raise NotImplementedError
+
+    @abstractmethod
+    def get_expression_type(self, node: Expression, type_context: Type | None = None) -> Type:
+        """Checks the type of the given expression."""
         raise NotImplementedError
 
 
@@ -244,15 +263,14 @@ class SemanticAnalyzerPluginInterface:
     # TODO: clean-up lookup functions.
     """
 
-    modules: Dict[str, MypyFile]
+    modules: dict[str, MypyFile]
     # Options for current file.
     options: Options
     cur_mod_id: str
     msg: MessageBuilder
 
     @abstractmethod
-    def named_type(self, fullname: str,
-                   args: Optional[List[Type]] = None) -> Instance:
+    def named_type(self, fullname: str, args: list[Type] | None = None) -> Instance:
         """Construct an instance of a builtin type with given type arguments."""
         raise NotImplementedError
 
@@ -263,8 +281,7 @@ class SemanticAnalyzerPluginInterface:
         raise NotImplementedError
 
     @abstractmethod
-    def named_type_or_none(self, fullname: str,
-                           args: Optional[List[Type]] = None) -> Optional[Instance]:
+    def named_type_or_none(self, fullname: str, args: list[Type] | None = None) -> Instance | None:
         """Construct an instance of a type with given type arguments.
 
         Return None if a type could not be constructed for the qualified
@@ -278,23 +295,38 @@ class SemanticAnalyzerPluginInterface:
         raise NotImplementedError
 
     @abstractmethod
-    def parse_bool(self, expr: Expression) -> Optional[bool]:
+    def parse_bool(self, expr: Expression) -> bool | None:
         """Parse True/False literals."""
         raise NotImplementedError
 
     @abstractmethod
-    def fail(self, msg: str, ctx: Context, serious: bool = False, *,
-             blocker: bool = False, code: Optional[ErrorCode] = None) -> None:
+    def parse_str_literal(self, expr: Expression) -> str | None:
+        """Parse string literals."""
+
+    @abstractmethod
+    def fail(
+        self,
+        msg: str,
+        ctx: Context,
+        serious: bool = False,
+        *,
+        blocker: bool = False,
+        code: ErrorCode | None = None,
+    ) -> None:
         """Emit an error message at given location."""
         raise NotImplementedError
 
     @abstractmethod
-    def anal_type(self, t: Type, *,
-                  tvar_scope: Optional[TypeVarLikeScope] = None,
-                  allow_tuple_literal: bool = False,
-                  allow_unbound_tvars: bool = False,
-                  report_invalid_types: bool = True,
-                  third_pass: bool = False) -> Optional[Type]:
+    def anal_type(
+        self,
+        typ: Type,
+        /,
+        *,
+        tvar_scope: TypeVarLikeScope | None = None,
+        allow_tuple_literal: bool = False,
+        allow_unbound_tvars: bool = False,
+        report_invalid_types: bool = True,
+    ) -> Type | None:
         """Analyze an unbound type.
 
         Return None if some part of the type is not ready yet. In this
@@ -309,7 +341,7 @@ class SemanticAnalyzerPluginInterface:
         raise NotImplementedError
 
     @abstractmethod
-    def lookup_fully_qualified(self, name: str) -> SymbolTableNode:
+    def lookup_fully_qualified(self, fullname: str, /) -> SymbolTableNode:
         """Lookup a symbol by its fully qualified name.
 
         Raise an error if not found.
@@ -317,7 +349,7 @@ class SemanticAnalyzerPluginInterface:
         raise NotImplementedError
 
     @abstractmethod
-    def lookup_fully_qualified_or_none(self, name: str) -> Optional[SymbolTableNode]:
+    def lookup_fully_qualified_or_none(self, fullname: str, /) -> SymbolTableNode | None:
         """Lookup a symbol by its fully qualified name.
 
         Return None if not found.
@@ -325,8 +357,9 @@ class SemanticAnalyzerPluginInterface:
         raise NotImplementedError
 
     @abstractmethod
-    def lookup_qualified(self, name: str, ctx: Context,
-                         suppress_errors: bool = False) -> Optional[SymbolTableNode]:
+    def lookup_qualified(
+        self, name: str, ctx: Context, suppress_errors: bool = False
+    ) -> SymbolTableNode | None:
         """Lookup symbol using a name in current scope.
 
         This follows Python local->non-local->global->builtins rules.
@@ -334,7 +367,7 @@ class SemanticAnalyzerPluginInterface:
         raise NotImplementedError
 
     @abstractmethod
-    def add_plugin_dependency(self, trigger: str, target: Optional[str] = None) -> None:
+    def add_plugin_dependency(self, trigger: str, target: str | None = None) -> None:
         """Specify semantic dependencies for generated methods/variables.
 
         If the symbol with full name given by trigger is found to be stale by mypy,
@@ -352,12 +385,12 @@ class SemanticAnalyzerPluginInterface:
         raise NotImplementedError
 
     @abstractmethod
-    def add_symbol_table_node(self, name: str, stnode: SymbolTableNode) -> Any:
+    def add_symbol_table_node(self, name: str, symbol: SymbolTableNode) -> Any:
         """Add node to global symbol table (or to nearest class if there is one)."""
         raise NotImplementedError
 
     @abstractmethod
-    def qualified_name(self, n: str) -> str:
+    def qualified_name(self, name: str) -> str:
         """Make qualified name using current module and enclosing class (if any)."""
         raise NotImplementedError
 
@@ -380,6 +413,10 @@ class SemanticAnalyzerPluginInterface:
     def is_stub_file(self) -> bool:
         raise NotImplementedError
 
+    @abstractmethod
+    def analyze_simple_literal_type(self, rvalue: Expression, is_final: bool) -> Type | None:
+        raise NotImplementedError
+
 
 # A context for querying for configuration data about a module for
 # cache invalidation purposes.
@@ -393,7 +430,7 @@ class ReportConfigContext(NamedTuple):
 # function.  Note that argument types aren't available yet.  If you need them,
 # you have to use a method hook instead.
 class FunctionSigContext(NamedTuple):
-    args: List[List[Expression]]  # Actual expressions for each formal argument
+    args: list[list[Expression]]  # Actual expressions for each formal argument
     default_signature: CallableType  # Original signature of the method
     context: Context  # Relevant location context (e.g. for error messages)
     api: CheckerPluginInterface
@@ -405,20 +442,20 @@ class FunctionSigContext(NamedTuple):
 # A no-op callback would just return the inferred return type, but a useful
 # callback at least sometimes can infer a more precise type.
 class FunctionContext(NamedTuple):
-    arg_types: List[List[Type]]  # List of actual caller types for each formal argument
-    arg_kinds: List[List[ArgKind]]  # Ditto for argument kinds, see nodes.ARG_* constants
+    arg_types: list[list[Type]]  # List of actual caller types for each formal argument
+    arg_kinds: list[list[ArgKind]]  # Ditto for argument kinds, see nodes.ARG_* constants
     # Names of formal parameters from the callee definition,
     # these will be sufficient in most cases.
-    callee_arg_names: List[Optional[str]]
+    callee_arg_names: list[str | None]
     # Names of actual arguments in the call expression. For example,
     # in a situation like this:
     #     def func(**kwargs) -> None:
     #         pass
     #     func(kw1=1, kw2=2)
     # callee_arg_names will be ['kwargs'] and arg_names will be [['kw1', 'kw2']].
-    arg_names: List[List[Optional[str]]]
+    arg_names: list[list[str | None]]
     default_return_type: Type  # Return type inferred from signature
-    args: List[List[Expression]]  # Actual expressions for each formal argument
+    args: list[list[Expression]]  # Actual expressions for each formal argument
     context: Context  # Relevant location context (e.g. for error messages)
     api: CheckerPluginInterface
 
@@ -429,7 +466,7 @@ class FunctionContext(NamedTuple):
 # TODO: document ProperType in the plugin changelog/update issue.
 class MethodSigContext(NamedTuple):
     type: ProperType  # Base object type for method call
-    args: List[List[Expression]]  # Actual expressions for each formal argument
+    args: list[list[Expression]]  # Actual expressions for each formal argument
     default_signature: CallableType  # Original signature of the method
     context: Context  # Relevant location context (e.g. for error messages)
     api: CheckerPluginInterface
@@ -441,13 +478,13 @@ class MethodSigContext(NamedTuple):
 # This is very similar to FunctionContext (only differences are documented).
 class MethodContext(NamedTuple):
     type: ProperType  # Base object type for method call
-    arg_types: List[List[Type]]  # List of actual caller types for each formal argument
+    arg_types: list[list[Type]]  # List of actual caller types for each formal argument
     # see FunctionContext for details about names and kinds
-    arg_kinds: List[List[ArgKind]]
-    callee_arg_names: List[Optional[str]]
-    arg_names: List[List[Optional[str]]]
+    arg_kinds: list[list[ArgKind]]
+    callee_arg_names: list[str | None]
+    arg_names: list[list[str | None]]
     default_return_type: Type  # Return type inferred by mypy
-    args: List[List[Expression]]  # Lists of actual expressions for every formal argument
+    args: list[list[Expression]]  # Lists of actual expressions for every formal argument
     context: Context
     api: CheckerPluginInterface
 
@@ -456,6 +493,7 @@ class MethodContext(NamedTuple):
 class AttributeContext(NamedTuple):
     type: ProperType  # Type of object with attribute
     default_attr_type: Type  # Original attribute type
+    is_lvalue: bool  # Whether the attribute is the target of an assignment
     context: Context  # Relevant location context (e.g. for error messages)
     api: CheckerPluginInterface
 
@@ -463,7 +501,7 @@ class AttributeContext(NamedTuple):
 # A context for a class hook that modifies the class definition.
 class ClassDefContext(NamedTuple):
     cls: ClassDef  # The class definition
-    reason: Expression   # The expression being applied (decorator, metaclass, base class)
+    reason: Expression  # The expression being applied (decorator, metaclass, base class)
     api: SemanticAnalyzerPluginInterface
 
 
@@ -496,12 +534,12 @@ class Plugin(CommonPluginApi):
         # This can't be set in __init__ because it is executed too soon in build.py.
         # Therefore, build.py *must* set it later before graph processing starts
         # by calling set_modules().
-        self._modules: Optional[Dict[str, MypyFile]] = None
+        self._modules: dict[str, MypyFile] | None = None
 
-    def set_modules(self, modules: Dict[str, MypyFile]) -> None:
+    def set_modules(self, modules: dict[str, MypyFile]) -> None:
         self._modules = modules
 
-    def lookup_fully_qualified(self, fullname: str) -> Optional[SymbolTableNode]:
+    def lookup_fully_qualified(self, fullname: str) -> SymbolTableNode | None:
         assert self._modules is not None
         return lookup_fully_qualified(fullname, self._modules)
 
@@ -528,7 +566,7 @@ class Plugin(CommonPluginApi):
         """
         return None
 
-    def get_additional_deps(self, file: MypyFile) -> List[Tuple[int, str, int]]:
+    def get_additional_deps(self, file: MypyFile) -> list[tuple[int, str, int]]:
         """Customize dependencies for a module.
 
         This hook allows adding in new dependencies for a module. It
@@ -545,8 +583,7 @@ class Plugin(CommonPluginApi):
         """
         return []
 
-    def get_type_analyze_hook(self, fullname: str
-                              ) -> Optional[Callable[[AnalyzeTypeContext], Type]]:
+    def get_type_analyze_hook(self, fullname: str) -> Callable[[AnalyzeTypeContext], Type] | None:
         """Customize behaviour of the type analyzer for given full names.
 
         This method is called during the semantic analysis pass whenever mypy sees an
@@ -564,8 +601,9 @@ class Plugin(CommonPluginApi):
         """
         return None
 
-    def get_function_signature_hook(self, fullname: str
-                                    ) -> Optional[Callable[[FunctionSigContext], FunctionLike]]:
+    def get_function_signature_hook(
+        self, fullname: str
+    ) -> Callable[[FunctionSigContext], FunctionLike] | None:
         """Adjust the signature of a function.
 
         This method is called before type checking a function call. Plugin
@@ -580,8 +618,7 @@ class Plugin(CommonPluginApi):
         """
         return None
 
-    def get_function_hook(self, fullname: str
-                          ) -> Optional[Callable[[FunctionContext], Type]]:
+    def get_function_hook(self, fullname: str) -> Callable[[FunctionContext], Type] | None:
         """Adjust the return type of a function call.
 
         This method is called after type checking a call. Plugin may adjust the return
@@ -597,8 +634,9 @@ class Plugin(CommonPluginApi):
         """
         return None
 
-    def get_method_signature_hook(self, fullname: str
-                                  ) -> Optional[Callable[[MethodSigContext], FunctionLike]]:
+    def get_method_signature_hook(
+        self, fullname: str
+    ) -> Callable[[MethodSigContext], FunctionLike] | None:
         """Adjust the signature of a method.
 
         This method is called before type checking a method call. Plugin
@@ -626,8 +664,7 @@ class Plugin(CommonPluginApi):
         """
         return None
 
-    def get_method_hook(self, fullname: str
-                        ) -> Optional[Callable[[MethodContext], Type]]:
+    def get_method_hook(self, fullname: str) -> Callable[[MethodContext], Type] | None:
         """Adjust return type of a method call.
 
         This is the same as get_function_hook(), but is called with the
@@ -635,8 +672,7 @@ class Plugin(CommonPluginApi):
         """
         return None
 
-    def get_attribute_hook(self, fullname: str
-                           ) -> Optional[Callable[[AttributeContext], Type]]:
+    def get_attribute_hook(self, fullname: str) -> Callable[[AttributeContext], Type] | None:
         """Adjust type of an instance attribute.
 
         This method is called with attribute full name using the class of the instance where
@@ -667,8 +703,7 @@ class Plugin(CommonPluginApi):
         """
         return None
 
-    def get_class_attribute_hook(self, fullname: str
-                                 ) -> Optional[Callable[[AttributeContext], Type]]:
+    def get_class_attribute_hook(self, fullname: str) -> Callable[[AttributeContext], Type] | None:
         """
         Adjust type of a class attribute.
 
@@ -686,8 +721,7 @@ class Plugin(CommonPluginApi):
         """
         return None
 
-    def get_class_decorator_hook(self, fullname: str
-                                 ) -> Optional[Callable[[ClassDefContext], None]]:
+    def get_class_decorator_hook(self, fullname: str) -> Callable[[ClassDefContext], None] | None:
         """Update class definition for given class decorators.
 
         The plugin can modify a TypeInfo _in place_ (for example add some generated
@@ -705,8 +739,9 @@ class Plugin(CommonPluginApi):
         """
         return None
 
-    def get_class_decorator_hook_2(self, fullname: str
-                                   ) -> Optional[Callable[[ClassDefContext], bool]]:
+    def get_class_decorator_hook_2(
+        self, fullname: str
+    ) -> Callable[[ClassDefContext], bool] | None:
         """Update class definition for given class decorators.
 
         Similar to get_class_decorator_hook, but this runs in a later pass when
@@ -722,8 +757,7 @@ class Plugin(CommonPluginApi):
         """
         return None
 
-    def get_metaclass_hook(self, fullname: str
-                           ) -> Optional[Callable[[ClassDefContext], None]]:
+    def get_metaclass_hook(self, fullname: str) -> Callable[[ClassDefContext], None] | None:
         """Update class definition for given declared metaclasses.
 
         Same as get_class_decorator_hook() but for metaclasses. Note:
@@ -734,8 +768,7 @@ class Plugin(CommonPluginApi):
         """
         return None
 
-    def get_base_class_hook(self, fullname: str
-                            ) -> Optional[Callable[[ClassDefContext], None]]:
+    def get_base_class_hook(self, fullname: str) -> Callable[[ClassDefContext], None] | None:
         """Update class definition for given base classes.
 
         Same as get_class_decorator_hook() but for base classes. Base classes
@@ -744,8 +777,9 @@ class Plugin(CommonPluginApi):
         """
         return None
 
-    def get_customize_class_mro_hook(self, fullname: str
-                                     ) -> Optional[Callable[[ClassDefContext], None]]:
+    def get_customize_class_mro_hook(
+        self, fullname: str
+    ) -> Callable[[ClassDefContext], None] | None:
         """Customize MRO for given classes.
 
         The plugin can modify the class MRO _in place_. This method is called
@@ -753,8 +787,9 @@ class Plugin(CommonPluginApi):
         """
         return None
 
-    def get_dynamic_class_hook(self, fullname: str
-                               ) -> Optional[Callable[[DynamicClassDefContext], None]]:
+    def get_dynamic_class_hook(
+        self, fullname: str
+    ) -> Callable[[DynamicClassDefContext], None] | None:
         """Semantically analyze a dynamic class definition.
 
         This plugin hook allows one to semantically analyze dynamic class definitions like:
@@ -770,7 +805,7 @@ class Plugin(CommonPluginApi):
         return None
 
 
-T = TypeVar('T')
+T = TypeVar("T")
 
 
 class ChainedPlugin(Plugin):
@@ -785,7 +820,7 @@ class ChainedPlugin(Plugin):
 
     # TODO: Support caching of lookup results (through a LRU cache, for example).
 
-    def __init__(self, options: Options, plugins: List[Plugin]) -> None:
+    def __init__(self, options: Options, plugins: list[Plugin]) -> None:
         """Initialize chained plugin.
 
         Assume that the child plugins aren't mutated (results may be cached).
@@ -793,7 +828,7 @@ class ChainedPlugin(Plugin):
         super().__init__(options)
         self._plugins = plugins
 
-    def set_modules(self, modules: Dict[str, MypyFile]) -> None:
+    def set_modules(self, modules: dict[str, MypyFile]) -> None:
         for plugin in self._plugins:
             plugin.set_modules(modules)
 
@@ -801,65 +836,62 @@ class ChainedPlugin(Plugin):
         config_data = [plugin.report_config_data(ctx) for plugin in self._plugins]
         return config_data if any(x is not None for x in config_data) else None
 
-    def get_additional_deps(self, file: MypyFile) -> List[Tuple[int, str, int]]:
+    def get_additional_deps(self, file: MypyFile) -> list[tuple[int, str, int]]:
         deps = []
         for plugin in self._plugins:
             deps.extend(plugin.get_additional_deps(file))
         return deps
 
-    def get_type_analyze_hook(self, fullname: str
-                              ) -> Optional[Callable[[AnalyzeTypeContext], Type]]:
+    def get_type_analyze_hook(self, fullname: str) -> Callable[[AnalyzeTypeContext], Type] | None:
         return self._find_hook(lambda plugin: plugin.get_type_analyze_hook(fullname))
 
-    def get_function_signature_hook(self, fullname: str
-                                    ) -> Optional[Callable[[FunctionSigContext], FunctionLike]]:
+    def get_function_signature_hook(
+        self, fullname: str
+    ) -> Callable[[FunctionSigContext], FunctionLike] | None:
         return self._find_hook(lambda plugin: plugin.get_function_signature_hook(fullname))
 
-    def get_function_hook(self, fullname: str
-                          ) -> Optional[Callable[[FunctionContext], Type]]:
+    def get_function_hook(self, fullname: str) -> Callable[[FunctionContext], Type] | None:
         return self._find_hook(lambda plugin: plugin.get_function_hook(fullname))
 
-    def get_method_signature_hook(self, fullname: str
-                                  ) -> Optional[Callable[[MethodSigContext], FunctionLike]]:
+    def get_method_signature_hook(
+        self, fullname: str
+    ) -> Callable[[MethodSigContext], FunctionLike] | None:
         return self._find_hook(lambda plugin: plugin.get_method_signature_hook(fullname))
 
-    def get_method_hook(self, fullname: str
-                        ) -> Optional[Callable[[MethodContext], Type]]:
+    def get_method_hook(self, fullname: str) -> Callable[[MethodContext], Type] | None:
         return self._find_hook(lambda plugin: plugin.get_method_hook(fullname))
 
-    def get_attribute_hook(self, fullname: str
-                           ) -> Optional[Callable[[AttributeContext], Type]]:
+    def get_attribute_hook(self, fullname: str) -> Callable[[AttributeContext], Type] | None:
         return self._find_hook(lambda plugin: plugin.get_attribute_hook(fullname))
 
-    def get_class_attribute_hook(self, fullname: str
-                                 ) -> Optional[Callable[[AttributeContext], Type]]:
+    def get_class_attribute_hook(self, fullname: str) -> Callable[[AttributeContext], Type] | None:
         return self._find_hook(lambda plugin: plugin.get_class_attribute_hook(fullname))
 
-    def get_class_decorator_hook(self, fullname: str
-                                 ) -> Optional[Callable[[ClassDefContext], None]]:
+    def get_class_decorator_hook(self, fullname: str) -> Callable[[ClassDefContext], None] | None:
         return self._find_hook(lambda plugin: plugin.get_class_decorator_hook(fullname))
 
-    def get_class_decorator_hook_2(self, fullname: str
-                                   ) -> Optional[Callable[[ClassDefContext], bool]]:
+    def get_class_decorator_hook_2(
+        self, fullname: str
+    ) -> Callable[[ClassDefContext], bool] | None:
         return self._find_hook(lambda plugin: plugin.get_class_decorator_hook_2(fullname))
 
-    def get_metaclass_hook(self, fullname: str
-                           ) -> Optional[Callable[[ClassDefContext], None]]:
+    def get_metaclass_hook(self, fullname: str) -> Callable[[ClassDefContext], None] | None:
         return self._find_hook(lambda plugin: plugin.get_metaclass_hook(fullname))
 
-    def get_base_class_hook(self, fullname: str
-                            ) -> Optional[Callable[[ClassDefContext], None]]:
+    def get_base_class_hook(self, fullname: str) -> Callable[[ClassDefContext], None] | None:
         return self._find_hook(lambda plugin: plugin.get_base_class_hook(fullname))
 
-    def get_customize_class_mro_hook(self, fullname: str
-                                     ) -> Optional[Callable[[ClassDefContext], None]]:
+    def get_customize_class_mro_hook(
+        self, fullname: str
+    ) -> Callable[[ClassDefContext], None] | None:
         return self._find_hook(lambda plugin: plugin.get_customize_class_mro_hook(fullname))
 
-    def get_dynamic_class_hook(self, fullname: str
-                               ) -> Optional[Callable[[DynamicClassDefContext], None]]:
+    def get_dynamic_class_hook(
+        self, fullname: str
+    ) -> Callable[[DynamicClassDefContext], None] | None:
         return self._find_hook(lambda plugin: plugin.get_dynamic_class_hook(fullname))
 
-    def _find_hook(self, lookup: Callable[[Plugin], T]) -> Optional[T]:
+    def _find_hook(self, lookup: Callable[[Plugin], T]) -> T | None:
         for plugin in self._plugins:
             hook = lookup(plugin)
             if hook:
