@@ -1,11 +1,32 @@
+from __future__ import annotations
+
+from collections.abc import Iterator
 from contextlib import contextmanager
-from typing import Dict, Iterator, List, Set
-from typing_extensions import Final
+from typing import Final
 
 from mypy.nodes import (
-    Block, AssignmentStmt, NameExpr, MypyFile, FuncDef, Lvalue, ListExpr, TupleExpr,
-    WhileStmt, ForStmt, BreakStmt, ContinueStmt, TryStmt, WithStmt, MatchStmt, StarExpr,
-    ImportFrom, MemberExpr, IndexExpr, Import, ImportAll, ClassDef
+    AssignmentStmt,
+    Block,
+    BreakStmt,
+    ClassDef,
+    ContinueStmt,
+    ForStmt,
+    FuncDef,
+    Import,
+    ImportAll,
+    ImportFrom,
+    IndexExpr,
+    ListExpr,
+    Lvalue,
+    MatchStmt,
+    MemberExpr,
+    MypyFile,
+    NameExpr,
+    StarExpr,
+    TryStmt,
+    TupleExpr,
+    WhileStmt,
+    WithStmt,
 )
 from mypy.patterns import AsPattern
 from mypy.traverser import TraverserVisitor
@@ -55,20 +76,20 @@ class VariableRenameVisitor(TraverserVisitor):
         # Number of surrounding loop statements
         self.loop_depth = 0
         # Map block id to loop depth.
-        self.block_loop_depth: Dict[int, int] = {}
+        self.block_loop_depth: dict[int, int] = {}
         # Stack of block ids being processed.
-        self.blocks: List[int] = []
+        self.blocks: list[int] = []
         # List of scopes; each scope maps short (unqualified) name to block id.
-        self.var_blocks: List[Dict[str, int]] = []
+        self.var_blocks: list[dict[str, int]] = []
 
         # References to variables that we may need to rename. List of
         # scopes; each scope is a mapping from name to list of collections
         # of names that refer to the same logical variable.
-        self.refs: List[Dict[str, List[List[NameExpr]]]] = []
+        self.refs: list[dict[str, list[list[NameExpr]]]] = []
         # Number of reads of the most recent definition of a variable (per scope)
-        self.num_reads: List[Dict[str, int]] = []
+        self.num_reads: list[dict[str, int]] = []
         # Kinds of nested scopes (FILE, FUNCTION or CLASS)
-        self.scope_kinds: List[int] = []
+        self.scope_kinds: list[int] = []
 
     def visit_mypy_file(self, file_node: MypyFile) -> None:
         """Rename variables within a file.
@@ -90,7 +111,7 @@ class VariableRenameVisitor(TraverserVisitor):
                 name = arg.variable.name
                 # 'self' can't be redefined since it's special as it allows definition of
                 # attributes. 'cls' can't be used to define attributes so we can ignore it.
-                can_be_redefined = name != 'self'  # TODO: Proper check
+                can_be_redefined = name != "self"  # TODO: Proper check
                 self.record_assignment(arg.variable.name, can_be_redefined)
                 self.handle_arg(name)
 
@@ -161,6 +182,7 @@ class VariableRenameVisitor(TraverserVisitor):
             self.analyze_lvalue(lvalue)
 
     def visit_match_stmt(self, s: MatchStmt) -> None:
+        s.subject.accept(self)
         for i in range(len(s.patterns)):
             with self.enter_block():
                 s.patterns[i].accept(self)
@@ -249,7 +271,7 @@ class VariableRenameVisitor(TraverserVisitor):
         This will be called at the end of a scope.
         """
         is_func = self.scope_kinds[-1] == FUNCTION
-        for name, refs in self.refs[-1].items():
+        for refs in self.refs[-1].values():
             if len(refs) == 1:
                 # Only one definition -- no renaming needed.
                 continue
@@ -421,14 +443,14 @@ class LimitedVariableRenameVisitor(TraverserVisitor):
     def __init__(self) -> None:
         # Short names of variables bound in with statements using "as"
         # in a surrounding scope
-        self.bound_vars: List[str] = []
+        self.bound_vars: list[str] = []
         # Stack of names that can't be safely renamed, per scope ('*' means that
         # no names can be renamed)
-        self.skipped: List[Set[str]] = []
+        self.skipped: list[set[str]] = []
         # References to variables that we may need to rename. Stack of
         # scopes; each scope is a mapping from name to list of collections
         # of names that refer to the same logical variable.
-        self.refs: List[Dict[str, List[List[NameExpr]]]] = []
+        self.refs: list[dict[str, list[list[NameExpr]]]] = []
 
     def visit_mypy_file(self, file_node: MypyFile) -> None:
         """Rename variables within a file.
@@ -521,7 +543,7 @@ class LimitedVariableRenameVisitor(TraverserVisitor):
         self.flush_refs()
 
     def reject_redefinition_of_vars_in_scope(self) -> None:
-        self.record_skipped('*')
+        self.record_skipped("*")
 
     def record_skipped(self, name: str) -> None:
         self.skipped[-1].add(name)
@@ -529,7 +551,7 @@ class LimitedVariableRenameVisitor(TraverserVisitor):
     def flush_refs(self) -> None:
         ref_dict = self.refs.pop()
         skipped = self.skipped.pop()
-        if '*' not in skipped:
+        if "*" not in skipped:
             for name, refs in ref_dict.items():
                 if len(refs) <= 1 or name in skipped:
                     continue
@@ -540,7 +562,7 @@ class LimitedVariableRenameVisitor(TraverserVisitor):
                     rename_refs(item, i)
 
 
-def rename_refs(names: List[NameExpr], index: int) -> None:
+def rename_refs(names: list[NameExpr], index: int) -> None:
     name = names[0].name
     new_name = name + "'" * (index + 1)
     for expr in names:
