@@ -24,36 +24,42 @@ deferral if they can't be satisfied. Initially every module in the SCC
 will be incomplete.
 """
 
-from typing import List, Tuple, Optional, Union, Callable
-from typing_extensions import TYPE_CHECKING, Final, TypeAlias as _TypeAlias
+from __future__ import annotations
 
-from mypy.backports import nullcontext
-from mypy.nodes import (
-    MypyFile, TypeInfo, FuncDef, Decorator, OverloadedFuncDef, Var
-)
-from mypy.semanal_typeargs import TypeArgumentAnalyzer
+from contextlib import nullcontext
+from typing import TYPE_CHECKING, Callable, Final, Optional, Union
+from typing_extensions import TypeAlias as _TypeAlias
+
+import mypy.build
 import mypy.state
-from mypy.semanal import (
-    SemanticAnalyzer, apply_semantic_analyzer_patches, remove_imported_names_from_symtable
-)
-from mypy.semanal_classprop import (
-    calculate_class_abstract_status, calculate_class_vars, check_protocol_status,
-    add_type_promotion
-)
-from mypy.errors import Errors
-from mypy.semanal_infer import infer_decorator_signature_if_simple
 from mypy.checker import FineGrainedDeferredNode
-from mypy.server.aststrip import SavedAttributes
-from mypy.util import is_typeshed_file
+from mypy.errors import Errors
+from mypy.nodes import Decorator, FuncDef, MypyFile, OverloadedFuncDef, TypeInfo, Var
 from mypy.options import Options
 from mypy.plugin import ClassDefContext
-import mypy.build
+from mypy.plugins import dataclasses as dataclasses_plugin
+from mypy.semanal import (
+    SemanticAnalyzer,
+    apply_semantic_analyzer_patches,
+    remove_imported_names_from_symtable,
+)
+from mypy.semanal_classprop import (
+    add_type_promotion,
+    calculate_class_abstract_status,
+    calculate_class_vars,
+    check_protocol_status,
+)
+from mypy.semanal_infer import infer_decorator_signature_if_simple
+from mypy.semanal_shared import find_dataclass_transform_spec
+from mypy.semanal_typeargs import TypeArgumentAnalyzer
+from mypy.server.aststrip import SavedAttributes
+from mypy.util import is_typeshed_file
 
 if TYPE_CHECKING:
     from mypy.build import Graph, State
 
 
-Patches: _TypeAlias = List[Tuple[int, Callable[[], None]]]
+Patches: _TypeAlias = list[tuple[int, Callable[[], None]]]
 
 
 # If we perform this many iterations, raise an exception since we are likely stuck.
@@ -62,10 +68,17 @@ MAX_ITERATIONS: Final = 20
 
 # Number of passes over core modules before going on to the rest of the builtin SCC.
 CORE_WARMUP: Final = 2
-core_modules: Final = ['typing', 'builtins', 'abc', 'collections']
+core_modules: Final = [
+    "typing",
+    "_collections_abc",
+    "builtins",
+    "abc",
+    "collections",
+    "collections.abc",
+]
 
 
-def semantic_analysis_for_scc(graph: 'Graph', scc: List[str], errors: Errors) -> None:
+def semantic_analysis_for_scc(graph: Graph, scc: list[str], errors: Errors) -> None:
     """Perform semantic analysis for all modules in a SCC (import cycle).
 
     Assume that reachability analysis has already been performed.
@@ -89,11 +102,11 @@ def semantic_analysis_for_scc(graph: 'Graph', scc: List[str], errors: Errors) ->
     calculate_class_properties(graph, scc, errors)
     check_blockers(graph, scc)
     # Clean-up builtins, so that TypeVar etc. are not accessible without importing.
-    if 'builtins' in scc:
-        cleanup_builtin_scc(graph['builtins'])
+    if "builtins" in scc:
+        cleanup_builtin_scc(graph["builtins"])
 
 
-def cleanup_builtin_scc(state: 'State') -> None:
+def cleanup_builtin_scc(state: State) -> None:
     """Remove imported names from builtins namespace.
 
     This way names imported from typing in builtins.pyi aren't available
@@ -102,14 +115,12 @@ def cleanup_builtin_scc(state: 'State') -> None:
     processing builtins.pyi itself.
     """
     assert state.tree is not None
-    remove_imported_names_from_symtable(state.tree.names, 'builtins')
+    remove_imported_names_from_symtable(state.tree.names, "builtins")
 
 
 def semantic_analysis_for_targets(
-        state: 'State',
-        nodes: List[FineGrainedDeferredNode],
-        graph: 'Graph',
-        saved_attrs: SavedAttributes) -> None:
+    state: State, nodes: list[FineGrainedDeferredNode], graph: Graph, saved_attrs: SavedAttributes
+) -> None:
     """Semantically analyze only selected nodes in a given module.
 
     This essentially mirrors the logic of semantic_analysis_for_scc()
@@ -130,8 +141,9 @@ def semantic_analysis_for_targets(
         if isinstance(n.node, MypyFile):
             # Already done above.
             continue
-        process_top_level_function(analyzer, state, state.id,
-                                   n.node.fullname, n.node, n.active_typeinfo, patches)
+        process_top_level_function(
+            analyzer, state, state.id, n.node.fullname, n.node, n.active_typeinfo, patches
+        )
     apply_semantic_analyzer_patches(patches)
     apply_class_plugin_hooks(graph, [state.id], state.manager.errors)
     check_type_arguments_in_targets(nodes, state, state.manager.errors)
@@ -148,16 +160,21 @@ def restore_saved_attrs(saved_attrs: SavedAttributes) -> None:
         # This needs to mimic the logic in SemanticAnalyzer.analyze_member_lvalue()
         # regarding the existing variable in class body or in a superclass:
         # If the attribute of self is not defined in superclasses, create a new Var.
-        if (existing is None or
-                # (An abstract Var is considered as not defined.)
-                (isinstance(existing.node, Var) and existing.node.is_abstract_var) or
-                # Also an explicit declaration on self creates a new Var unless
-                # there is already one defined in the class body.
-                sym.node.explicit_self_type and not defined_in_this_class):
+        if (
+            existing is None
+            or
+            # (An abstract Var is considered as not defined.)
+            (isinstance(existing.node, Var) and existing.node.is_abstract_var)
+            or
+            # Also an explicit declaration on self creates a new Var unless
+            # there is already one defined in the class body.
+            sym.node.explicit_self_type
+            and not defined_in_this_class
+        ):
             info.names[name] = sym
 
 
-def process_top_levels(graph: 'Graph', scc: List[str], patches: Patches) -> None:
+def process_top_levels(graph: Graph, scc: list[str], patches: Patches) -> None:
     # Process top levels until everything has been bound.
 
     # Reverse order of the scc so the first modules in the original list will be
@@ -173,7 +190,7 @@ def process_top_levels(graph: 'Graph', scc: List[str], patches: Patches) -> None
     # Initially all namespaces in the SCC are incomplete (well they are empty).
     state.manager.incomplete_namespaces.update(scc)
 
-    worklist = scc[:]
+    worklist = scc.copy()
     # HACK: process core stuff first. This is mostly needed to support defining
     # named tuples in builtin SCC.
     if all(m in worklist for m in core_modules):
@@ -194,30 +211,28 @@ def process_top_levels(graph: 'Graph', scc: List[str], patches: Patches) -> None
         if final_iteration:
             # Give up. It's impossible to bind all names.
             state.manager.incomplete_namespaces.clear()
-        all_deferred: List[str] = []
+        all_deferred: list[str] = []
         any_progress = False
         while worklist:
             next_id = worklist.pop()
             state = graph[next_id]
             assert state.tree is not None
-            deferred, incomplete, progress = semantic_analyze_target(next_id, state,
-                                                                     state.tree,
-                                                                     None,
-                                                                     final_iteration,
-                                                                     patches)
+            deferred, incomplete, progress = semantic_analyze_target(
+                next_id, next_id, state, state.tree, None, final_iteration, patches
+            )
             all_deferred += deferred
             any_progress = any_progress or progress
             if not incomplete:
                 state.manager.incomplete_namespaces.discard(next_id)
         if final_iteration:
-            assert not all_deferred, 'Must not defer during final iteration'
+            assert not all_deferred, "Must not defer during final iteration"
         # Reverse to process the targets in the same order on every iteration. This avoids
         # processing the same target twice in a row, which is inefficient.
         worklist = list(reversed(all_deferred))
         final_iteration = not any_progress
 
 
-def process_functions(graph: 'Graph', scc: List[str], patches: Patches) -> None:
+def process_functions(graph: Graph, scc: list[str], patches: Patches) -> None:
     # Process functions.
     for module in scc:
         tree = graph[module].tree
@@ -234,22 +249,20 @@ def process_functions(graph: 'Graph', scc: List[str], patches: Patches) -> None:
         targets = sorted(get_all_leaf_targets(tree), key=lambda x: (x[1].line, x[0]))
         for target, node, active_type in targets:
             assert isinstance(node, (FuncDef, OverloadedFuncDef, Decorator))
-            process_top_level_function(analyzer,
-                                       graph[module],
-                                       module,
-                                       target,
-                                       node,
-                                       active_type,
-                                       patches)
+            process_top_level_function(
+                analyzer, graph[module], module, target, node, active_type, patches
+            )
 
 
-def process_top_level_function(analyzer: 'SemanticAnalyzer',
-                               state: 'State',
-                               module: str,
-                               target: str,
-                               node: Union[FuncDef, OverloadedFuncDef, Decorator],
-                               active_type: Optional[TypeInfo],
-                               patches: Patches) -> None:
+def process_top_level_function(
+    analyzer: SemanticAnalyzer,
+    state: State,
+    module: str,
+    target: str,
+    node: FuncDef | OverloadedFuncDef | Decorator,
+    active_type: TypeInfo | None,
+    patches: Patches,
+) -> None:
     """Analyze single top-level function or method.
 
     Process the body of the function (including nested functions) again and again,
@@ -275,10 +288,13 @@ def process_top_level_function(analyzer: 'SemanticAnalyzer',
         if not (deferred or incomplete) or final_iteration:
             # OK, this is one last pass, now missing names will be reported.
             analyzer.incomplete_namespaces.discard(module)
-        deferred, incomplete, progress = semantic_analyze_target(target, state, node, active_type,
-                                                                 final_iteration, patches)
+        deferred, incomplete, progress = semantic_analyze_target(
+            target, module, state, node, active_type, final_iteration, patches
+        )
+        if not incomplete:
+            state.manager.incomplete_namespaces.discard(module)
         if final_iteration:
-            assert not deferred, 'Must not defer during final iteration'
+            assert not deferred, "Must not defer during final iteration"
         if not progress:
             final_iteration = True
 
@@ -288,24 +304,29 @@ def process_top_level_function(analyzer: 'SemanticAnalyzer',
     analyzer.saved_locals.clear()
 
 
-TargetInfo = Tuple[str, Union[MypyFile, FuncDef, OverloadedFuncDef, Decorator], Optional[TypeInfo]]
+TargetInfo: _TypeAlias = tuple[
+    str, Union[MypyFile, FuncDef, OverloadedFuncDef, Decorator], Optional[TypeInfo]
+]
 
 
-def get_all_leaf_targets(file: MypyFile) -> List[TargetInfo]:
+def get_all_leaf_targets(file: MypyFile) -> list[TargetInfo]:
     """Return all leaf targets in a symbol table (module-level and methods)."""
-    result: List[TargetInfo] = []
+    result: list[TargetInfo] = []
     for fullname, node, active_type in file.local_definitions():
         if isinstance(node.node, (FuncDef, OverloadedFuncDef, Decorator)):
             result.append((fullname, node.node, active_type))
     return result
 
 
-def semantic_analyze_target(target: str,
-                            state: 'State',
-                            node: Union[MypyFile, FuncDef, OverloadedFuncDef, Decorator],
-                            active_type: Optional[TypeInfo],
-                            final_iteration: bool,
-                            patches: Patches) -> Tuple[List[str], bool, bool]:
+def semantic_analyze_target(
+    target: str,
+    module: str,
+    state: State,
+    node: MypyFile | FuncDef | OverloadedFuncDef | Decorator,
+    active_type: TypeInfo | None,
+    final_iteration: bool,
+    patches: Patches,
+) -> tuple[list[str], bool, bool]:
     """Semantically analyze a single target.
 
     Return tuple with these items:
@@ -313,7 +334,7 @@ def semantic_analyze_target(target: str,
     - was some definition incomplete (need to run another pass)
     - were any new names defined (or placeholders replaced)
     """
-    state.manager.processed_targets.append(target)
+    state.manager.processed_targets.append((module, target))
     tree = state.tree
     assert tree is not None
     analyzer = state.manager.semantic_analyzer
@@ -327,12 +348,14 @@ def semantic_analyze_target(target: str,
         if isinstance(refresh_node, Decorator):
             # Decorator expressions will be processed as part of the module top level.
             refresh_node = refresh_node.func
-        analyzer.refresh_partial(refresh_node,
-                                 patches,
-                                 final_iteration,
-                                 file_node=tree,
-                                 options=state.options,
-                                 active_type=active_type)
+        analyzer.refresh_partial(
+            refresh_node,
+            patches,
+            final_iteration,
+            file_node=tree,
+            options=state.options,
+            active_type=active_type,
+        )
         if isinstance(node, Decorator):
             infer_decorator_signature_if_simple(node, analyzer)
     for dep in analyzer.imports:
@@ -352,32 +375,39 @@ def semantic_analyze_target(target: str,
         return [], analyzer.incomplete, analyzer.progress
 
 
-def check_type_arguments(graph: 'Graph', scc: List[str], errors: Errors) -> None:
+def check_type_arguments(graph: Graph, scc: list[str], errors: Errors) -> None:
     for module in scc:
         state = graph[module]
         assert state.tree
-        analyzer = TypeArgumentAnalyzer(errors,
-                                        state.options,
-                                        is_typeshed_file(state.path or ''))
+        analyzer = TypeArgumentAnalyzer(
+            errors,
+            state.options,
+            state.tree.is_typeshed_file(state.options),
+            state.manager.semantic_analyzer.named_type,
+        )
         with state.wrap_context():
             with mypy.state.state.strict_optional_set(state.options.strict_optional):
                 state.tree.accept(analyzer)
 
 
-def check_type_arguments_in_targets(targets: List[FineGrainedDeferredNode], state: 'State',
-                                    errors: Errors) -> None:
+def check_type_arguments_in_targets(
+    targets: list[FineGrainedDeferredNode], state: State, errors: Errors
+) -> None:
     """Check type arguments against type variable bounds and restrictions.
 
     This mirrors the logic in check_type_arguments() except that we process only
     some targets. This is used in fine grained incremental mode.
     """
-    analyzer = TypeArgumentAnalyzer(errors,
-                                    state.options,
-                                    is_typeshed_file(state.path or ''))
+    analyzer = TypeArgumentAnalyzer(
+        errors,
+        state.options,
+        is_typeshed_file(state.options.abs_custom_typeshed_dir, state.path or ""),
+        state.manager.semantic_analyzer.named_type,
+    )
     with state.wrap_context():
         with mypy.state.state.strict_optional_set(state.options.strict_optional):
             for target in targets:
-                func: Optional[Union[FuncDef, OverloadedFuncDef]] = None
+                func: FuncDef | OverloadedFuncDef | None = None
                 if isinstance(target.node, (FuncDef, OverloadedFuncDef)):
                     func = target.node
                 saved = (state.id, target.active_typeinfo, func)  # module, class, function
@@ -386,7 +416,7 @@ def check_type_arguments_in_targets(targets: List[FineGrainedDeferredNode], stat
                     target.node.accept(analyzer)
 
 
-def apply_class_plugin_hooks(graph: 'Graph', scc: List[str], errors: Errors) -> None:
+def apply_class_plugin_hooks(graph: Graph, scc: list[str], errors: Errors) -> None:
     """Apply class plugin hooks within a SCC.
 
     We run these after to the main semantic analysis so that the hooks
@@ -410,32 +440,58 @@ def apply_class_plugin_hooks(graph: 'Graph', scc: List[str], errors: Errors) ->
             assert tree
             for _, node, _ in tree.local_definitions():
                 if isinstance(node.node, TypeInfo):
-                    if not apply_hooks_to_class(state.manager.semantic_analyzer,
-                                                module, node.node, state.options, tree, errors):
+                    if not apply_hooks_to_class(
+                        state.manager.semantic_analyzer,
+                        module,
+                        node.node,
+                        state.options,
+                        tree,
+                        errors,
+                    ):
                         incomplete = True
 
 
-def apply_hooks_to_class(self: SemanticAnalyzer,
-                         module: str,
-                         info: TypeInfo,
-                         options: Options,
-                         file_node: MypyFile,
-                         errors: Errors) -> bool:
+def apply_hooks_to_class(
+    self: SemanticAnalyzer,
+    module: str,
+    info: TypeInfo,
+    options: Options,
+    file_node: MypyFile,
+    errors: Errors,
+) -> bool:
     # TODO: Move more class-related hooks here?
     defn = info.defn
     ok = True
     for decorator in defn.decorators:
         with self.file_context(file_node, options, info):
+            hook = None
+
             decorator_name = self.get_fullname_for_hook(decorator)
             if decorator_name:
                 hook = self.plugin.get_class_decorator_hook_2(decorator_name)
-                if hook:
-                    ok = ok and hook(ClassDefContext(defn, decorator, self))
+            # Special case: if the decorator is itself decorated with
+            # typing.dataclass_transform, apply the hook for the dataclasses plugin
+            # TODO: remove special casing here
+            if hook is None and find_dataclass_transform_spec(decorator):
+                hook = dataclasses_plugin.dataclass_class_maker_callback
+
+            if hook:
+                ok = ok and hook(ClassDefContext(defn, decorator, self))
+
+    # Check if the class definition itself triggers a dataclass transform (via a parent class/
+    # metaclass)
+    spec = find_dataclass_transform_spec(info)
+    if spec is not None:
+        with self.file_context(file_node, options, info):
+            # We can't use the normal hook because reason = defn, and ClassDefContext only accepts
+            # an Expression for reason
+            ok = ok and dataclasses_plugin.DataclassTransformer(defn, defn, spec, self).transform()
+
     return ok
 
 
-def calculate_class_properties(graph: 'Graph', scc: List[str], errors: Errors) -> None:
-    builtins = graph['builtins'].tree
+def calculate_class_properties(graph: Graph, scc: list[str], errors: Errors) -> None:
+    builtins = graph["builtins"].tree
     assert builtins
     for module in scc:
         state = graph[module]
@@ -447,10 +503,11 @@ def calculate_class_properties(graph: 'Graph', scc: List[str], errors: Errors) -
                     calculate_class_abstract_status(node.node, tree.is_stub, errors)
                     check_protocol_status(node.node, errors)
                     calculate_class_vars(node.node)
-                    add_type_promotion(node.node, tree.names, graph[module].options,
-                                       builtins.names)
+                    add_type_promotion(
+                        node.node, tree.names, graph[module].options, builtins.names
+                    )
 
 
-def check_blockers(graph: 'Graph', scc: List[str]) -> None:
+def check_blockers(graph: Graph, scc: list[str]) -> None:
     for module in scc:
         graph[module].check_blockers()
