@@ -22,94 +22,116 @@ Other things:
  * No understanding of type variables at *all*
 """
 
-from typing import (
-    List, Optional, Tuple, Dict, Callable, Union, NamedTuple, TypeVar, Iterator, cast,
-)
-from typing_extensions import TypedDict
-
-from mypy.state import state
-from mypy.types import (
-    Type, AnyType, TypeOfAny, CallableType, UnionType, NoneType, Instance, TupleType,
-    TypeVarType, FunctionLike, UninhabitedType,
-    TypeStrVisitor, TypeTranslator,
-    is_optional, remove_optional, ProperType, get_proper_type,
-    TypedDictType, TypeAliasType
-)
-from mypy.build import State, Graph
-from mypy.nodes import (
-    ArgKind, ARG_STAR, ARG_STAR2, FuncDef, MypyFile, SymbolTable,
-    Decorator, RefExpr,
-    SymbolNode, TypeInfo, Expression, ReturnStmt, CallExpr,
-    reverse_builtin_aliases,
-)
-from mypy.server.update import FineGrainedBuildManager
-from mypy.util import split_target
-from mypy.find_sources import SourceFinder, InvalidSourceList
-from mypy.modulefinder import PYTHON_EXTENSIONS
-from mypy.plugin import Plugin, FunctionContext, MethodContext
-from mypy.traverser import TraverserVisitor
-from mypy.checkexpr import has_any_type, map_actuals_to_formals
-
-from mypy.join import join_type_list
-from mypy.meet import meet_type_list
-from mypy.sametypes import is_same_type
-from mypy.typeops import make_simplified_union
-
-from contextlib import contextmanager
+from __future__ import annotations
 
 import itertools
 import json
 import os
+from collections.abc import Iterator
+from contextlib import contextmanager
+from typing import Callable, NamedTuple, TypedDict, TypeVar, cast
+
+from mypy.argmap import map_actuals_to_formals
+from mypy.build import Graph, State
+from mypy.checkexpr import has_any_type
+from mypy.find_sources import InvalidSourceList, SourceFinder
+from mypy.join import join_type_list
+from mypy.meet import meet_type_list
+from mypy.modulefinder import PYTHON_EXTENSIONS
+from mypy.nodes import (
+    ARG_STAR,
+    ARG_STAR2,
+    ArgKind,
+    CallExpr,
+    Decorator,
+    Expression,
+    FuncDef,
+    MypyFile,
+    RefExpr,
+    ReturnStmt,
+    SymbolNode,
+    SymbolTable,
+    TypeInfo,
+    reverse_builtin_aliases,
+)
+from mypy.options import Options
+from mypy.plugin import FunctionContext, MethodContext, Plugin
+from mypy.server.update import FineGrainedBuildManager
+from mypy.state import state
+from mypy.traverser import TraverserVisitor
+from mypy.typeops import make_simplified_union
+from mypy.types import (
+    AnyType,
+    CallableType,
+    FunctionLike,
+    Instance,
+    NoneType,
+    ProperType,
+    TupleType,
+    Type,
+    TypeAliasType,
+    TypedDictType,
+    TypeOfAny,
+    TypeStrVisitor,
+    TypeTranslator,
+    TypeVarType,
+    UninhabitedType,
+    UnionType,
+    get_proper_type,
+)
+from mypy.types_utils import is_overlapping_none, remove_optional
+from mypy.util import split_target
 
 
 class PyAnnotateSignature(TypedDict):
     return_type: str
-    arg_types: List[str]
+    arg_types: list[str]
 
 
 class Callsite(NamedTuple):
     path: str
     line: int
-    arg_kinds: List[List[ArgKind]]
-    callee_arg_names: List[Optional[str]]
-    arg_names: List[List[Optional[str]]]
-    arg_types: List[List[Type]]
+    arg_kinds: list[list[ArgKind]]
+    callee_arg_names: list[str | None]
+    arg_names: list[list[str | None]]
+    arg_types: list[list[Type]]
 
 
 class SuggestionPlugin(Plugin):
     """Plugin that records all calls to a given target."""
 
     def __init__(self, target: str) -> None:
-        if target.endswith(('.__new__', '.__init__')):
-            target = target.rsplit('.', 1)[0]
+        if target.endswith((".__new__", ".__init__")):
+            target = target.rsplit(".", 1)[0]
 
         self.target = target
         # List of call sites found by dmypy suggest:
         # (path, line, <arg kinds>, <arg names>, <arg types>)
-        self.mystery_hits: List[Callsite] = []
+        self.mystery_hits: list[Callsite] = []
 
-    def get_function_hook(self, fullname: str
-                          ) -> Optional[Callable[[FunctionContext], Type]]:
+    def get_function_hook(self, fullname: str) -> Callable[[FunctionContext], Type] | None:
         if fullname == self.target:
             return self.log
         else:
             return None
 
-    def get_method_hook(self, fullname: str
-                        ) -> Optional[Callable[[MethodContext], Type]]:
+    def get_method_hook(self, fullname: str) -> Callable[[MethodContext], Type] | None:
         if fullname == self.target:
             return self.log
         else:
             return None
 
-    def log(self, ctx: Union[FunctionContext, MethodContext]) -> Type:
-        self.mystery_hits.append(Callsite(
-            ctx.api.path,
-            ctx.context.line,
-            ctx.arg_kinds,
-            ctx.callee_arg_names,
-            ctx.arg_names,
-            ctx.arg_types))
+    def log(self, ctx: FunctionContext | MethodContext) -> Type:
+        self.mystery_hits.append(
+            Callsite(
+                ctx.api.path,
+                ctx.context.line,
+                ctx.arg_kinds,
+                ctx.callee_arg_names,
+                ctx.arg_names,
+                ctx.arg_types,
+            )
+        )
         return ctx.default_return_type
 
 
@@ -117,9 +139,10 @@ class SuggestionPlugin(Plugin):
 # traversing into expressions
 class ReturnFinder(TraverserVisitor):
     """Visitor for finding all types returned from a function."""
-    def __init__(self, typemap: Dict[Expression, Type]) -> None:
+
+    def __init__(self, typemap: dict[Expression, Type]) -> None:
         self.typemap = typemap
-        self.return_types: List[Type] = []
+        self.return_types: list[Type] = []
 
     def visit_return_stmt(self, o: ReturnStmt) -> None:
         if o.expr is not None and o.expr in self.typemap:
@@ -130,7 +153,7 @@ class ReturnFinder(TraverserVisitor):
         pass
 
 
-def get_return_types(typemap: Dict[Expression, Type], func: FuncDef) -> List[Type]:
+def get_return_types(typemap: dict[Expression, Type], func: FuncDef) -> list[Type]:
     """Find all the types returned by return statements in func."""
     finder = ReturnFinder(typemap)
     func.body.accept(finder)
@@ -142,9 +165,10 @@ class ArgUseFinder(TraverserVisitor):
 
     This is extremely simple minded but might be effective anyways.
     """
-    def __init__(self, func: FuncDef, typemap: Dict[Expression, Type]) -> None:
+
+    def __init__(self, func: FuncDef, typemap: dict[Expression, Type]) -> None:
         self.typemap = typemap
-        self.arg_types: Dict[SymbolNode, List[Type]] = {arg.variable: [] for arg in func.arguments}
+        self.arg_types: dict[SymbolNode, list[Type]] = {arg.variable: [] for arg in func.arguments}
 
     def visit_call_expr(self, o: CallExpr) -> None:
         if not any(isinstance(e, RefExpr) and e.node in self.arg_types for e in o.args):
@@ -155,8 +179,12 @@ class ArgUseFinder(TraverserVisitor):
             return
 
         formal_to_actual = map_actuals_to_formals(
-            o.arg_kinds, o.arg_names, typ.arg_kinds, typ.arg_names,
-            lambda n: AnyType(TypeOfAny.special_form))
+            o.arg_kinds,
+            o.arg_names,
+            typ.arg_kinds,
+            typ.arg_names,
+            lambda n: AnyType(TypeOfAny.special_form),
+        )
 
         for i, args in enumerate(formal_to_actual):
             for arg_idx in args:
@@ -165,7 +193,7 @@ class ArgUseFinder(TraverserVisitor):
                     self.arg_types[arg.node].append(typ.arg_types[i])
 
 
-def get_arg_uses(typemap: Dict[Expression, Type], func: FuncDef) -> List[List[Type]]:
+def get_arg_uses(typemap: dict[Expression, Type], func: FuncDef) -> list[list[Type]]:
     """Find all the types of arguments that each arg is passed to.
 
     For example, given
@@ -204,16 +232,17 @@ def is_implicit_any(typ: Type) -> bool:
 class SuggestionEngine:
     """Engine for finding call sites and suggesting signatures."""
 
-    def __init__(self, fgmanager: FineGrainedBuildManager,
-                 *,
-                 json: bool,
-                 no_errors: bool = False,
-                 no_any: bool = False,
-                 try_text: bool = False,
-                 flex_any: Optional[float] = None,
-                 use_fixme: Optional[str] = None,
-                 max_guesses: Optional[int] = None
-                 ) -> None:
+    def __init__(
+        self,
+        fgmanager: FineGrainedBuildManager,
+        *,
+        json: bool,
+        no_errors: bool = False,
+        no_any: bool = False,
+        flex_any: float | None = None,
+        use_fixme: str | None = None,
+        max_guesses: int | None = None,
+    ) -> None:
         self.fgmanager = fgmanager
         self.manager = fgmanager.manager
         self.plugin = self.manager.plugin
@@ -222,7 +251,6 @@ class SuggestionEngine:
 
         self.give_json = json
         self.no_errors = no_errors
-        self.try_text = try_text
         self.flex_any = flex_any
         if no_any:
             self.flex_any = 1.0
@@ -249,10 +277,14 @@ class SuggestionEngine:
         with self.restore_after(mod):
             callsites, _ = self.get_callsites(node)
 
-        return '\n'.join(dedup(
-            [f"{path}:{line}: {self.format_args(arg_kinds, arg_names, arg_types)}"
-             for path, line, arg_kinds, _, arg_names, arg_types in callsites]
-        ))
+        return "\n".join(
+            dedup(
+                [
+                    f"{path}:{line}: {self.format_args(arg_kinds, arg_names, arg_types)}"
+                    for path, line, arg_kinds, _, arg_names, arg_types in callsites
+                ]
+            )
+        )
 
     @contextmanager
     def restore_after(self, module: str) -> Iterator[None]:
@@ -284,11 +316,12 @@ class SuggestionEngine:
         # since they need some special treatment (specifically,
         # constraint generation ignores them.)
         return CallableType(
-            [AnyType(TypeOfAny.suggestion_engine) for a in fdef.arg_kinds],
+            [AnyType(TypeOfAny.suggestion_engine) for _ in fdef.arg_kinds],
             fdef.arg_kinds,
             fdef.arg_names,
             AnyType(TypeOfAny.suggestion_engine),
-            self.named_type('builtins.function'))
+            self.named_type("builtins.function"),
+        )
 
     def get_starting_type(self, fdef: FuncDef) -> CallableType:
         if isinstance(fdef.type, CallableType):
@@ -296,12 +329,16 @@ class SuggestionEngine:
         else:
             return self.get_trivial_type(fdef)
 
-    def get_args(self, is_method: bool,
-                 base: CallableType, defaults: List[Optional[Type]],
-                 callsites: List[Callsite],
-                 uses: List[List[Type]]) -> List[List[Type]]:
+    def get_args(
+        self,
+        is_method: bool,
+        base: CallableType,
+        defaults: list[Type | None],
+        callsites: list[Callsite],
+        uses: list[list[Type]],
+    ) -> list[list[Type]]:
         """Produce a list of type suggestions for each argument type."""
-        types: List[List[Type]] = []
+        types: list[list[Type]] = []
         for i in range(len(base.arg_kinds)):
             # Make self args Any but this will get overridden somewhere in the checker
             if i == 0 and is_method:
@@ -328,10 +365,12 @@ class SuggestionEngine:
 
             arg_types = []
 
-            if (all_arg_types
-                    and all(isinstance(get_proper_type(tp), NoneType) for tp in all_arg_types)):
+            if all_arg_types and all(
+                isinstance(get_proper_type(tp), NoneType) for tp in all_arg_types
+            ):
                 arg_types.append(
-                    UnionType.make_union([all_arg_types[0], AnyType(TypeOfAny.explicit)]))
+                    UnionType.make_union([all_arg_types[0], AnyType(TypeOfAny.explicit)])
+                )
             elif all_arg_types:
                 arg_types.extend(generate_type_combinations(all_arg_types))
             else:
@@ -344,33 +383,31 @@ class SuggestionEngine:
             types.append(arg_types)
         return types
 
-    def get_default_arg_types(self, fdef: FuncDef) -> List[Optional[Type]]:
+    def get_default_arg_types(self, fdef: FuncDef) -> list[Type | None]:
         return [
             self.manager.all_types[arg.initializer] if arg.initializer else None
             for arg in fdef.arguments
         ]
 
-    def add_adjustments(self, typs: List[Type]) -> List[Type]:
-        if not self.try_text or self.manager.options.python_version[0] != 2:
-            return typs
-        translator = StrToText(self.named_type)
-        return dedup(typs + [tp.accept(translator) for tp in typs])
-
-    def get_guesses(self, is_method: bool, base: CallableType, defaults: List[Optional[Type]],
-                    callsites: List[Callsite],
-                    uses: List[List[Type]]) -> List[CallableType]:
+    def get_guesses(
+        self,
+        is_method: bool,
+        base: CallableType,
+        defaults: list[Type | None],
+        callsites: list[Callsite],
+        uses: list[list[Type]],
+    ) -> list[CallableType]:
         """Compute a list of guesses for a function's type.
 
         This focuses just on the argument types, and doesn't change the provided return type.
         """
         options = self.get_args(is_method, base, defaults, callsites, uses)
-        options = [self.add_adjustments(tps) for tps in options]
 
         # Take the first `max_guesses` guesses.
         product = itertools.islice(itertools.product(*options), 0, self.max_guesses)
         return [refine_callable(base, base.copy_modified(arg_types=list(x))) for x in product]
 
-    def get_callsites(self, func: FuncDef) -> Tuple[List[Callsite], List[str]]:
+    def get_callsites(self, func: FuncDef) -> tuple[list[Callsite], list[str]]:
         """Find all call sites of a function."""
         new_type = self.get_starting_type(func)
 
@@ -385,18 +422,19 @@ class SuggestionEngine:
         return collector_plugin.mystery_hits, errors
 
     def filter_options(
-        self, guesses: List[CallableType], is_method: bool, ignore_return: bool
-    ) -> List[CallableType]:
+        self, guesses: list[CallableType], is_method: bool, ignore_return: bool
+    ) -> list[CallableType]:
         """Apply any configured filters to the possible guesses.
 
         Currently the only option is filtering based on Any prevalance."""
         return [
-            t for t in guesses
+            t
+            for t in guesses
             if self.flex_any is None
             or any_score_callable(t, is_method, ignore_return) >= self.flex_any
         ]
 
-    def find_best(self, func: FuncDef, guesses: List[CallableType]) -> Tuple[CallableType, int]:
+    def find_best(self, func: FuncDef, guesses: list[CallableType]) -> tuple[CallableType, int]:
         """From a list of possible function types, find the best one.
 
         For best, we want the fewest errors, then the best "score" from score_callable.
@@ -404,11 +442,10 @@ class SuggestionEngine:
         if not guesses:
             raise SuggestionFailure("No guesses that match criteria!")
         errors = {guess: self.try_type(func, guess) for guess in guesses}
-        best = min(guesses,
-                   key=lambda s: (count_errors(errors[s]), self.score_callable(s)))
+        best = min(guesses, key=lambda s: (count_errors(errors[s]), self.score_callable(s)))
         return best, count_errors(errors[best])
 
-    def get_guesses_from_parent(self, node: FuncDef) -> List[CallableType]:
+    def get_guesses_from_parent(self, node: FuncDef) -> list[CallableType]:
         """Try to get a guess of a method type from a parent class."""
         if not node.info:
             return []
@@ -469,25 +506,27 @@ class SuggestionEngine:
 
         return self.pyannotate_signature(mod, is_method, best)
 
-    def format_args(self,
-                    arg_kinds: List[List[ArgKind]],
-                    arg_names: List[List[Optional[str]]],
-                    arg_types: List[List[Type]]) -> str:
-        args: List[str] = []
+    def format_args(
+        self,
+        arg_kinds: list[list[ArgKind]],
+        arg_names: list[list[str | None]],
+        arg_types: list[list[Type]],
+    ) -> str:
+        args: list[str] = []
         for i in range(len(arg_types)):
             for kind, name, typ in zip(arg_kinds[i], arg_names[i], arg_types[i]):
                 arg = self.format_type(None, typ)
                 if kind == ARG_STAR:
-                    arg = '*' + arg
+                    arg = "*" + arg
                 elif kind == ARG_STAR2:
-                    arg = '**' + arg
+                    arg = "**" + arg
                 elif kind.is_named():
                     if name:
                         arg = f"{name}={arg}"
             args.append(arg)
         return f"({', '.join(args)})"
 
-    def find_node(self, key: str) -> Tuple[str, str, FuncDef]:
+    def find_node(self, key: str) -> tuple[str, str, FuncDef]:
         """From a target name, return module/target names and the func def.
 
         The 'key' argument can be in one of two formats:
@@ -496,18 +535,19 @@ class SuggestionEngine:
           e.g., path/to/file.py:42
         """
         # TODO: Also return OverloadedFuncDef -- currently these are ignored.
-        node: Optional[SymbolNode] = None
-        if ':' in key:
-            if key.count(':') > 1:
+        node: SymbolNode | None = None
+        if ":" in key:
+            if key.count(":") > 1:
                 raise SuggestionFailure(
-                    'Malformed location for function: {}. Must be either'
-                    ' package.module.Class.method or path/to/file.py:line'.format(key))
-            file, line = key.split(':')
+                    "Malformed location for function: {}. Must be either"
+                    " package.module.Class.method or path/to/file.py:line".format(key)
+                )
+            file, line = key.split(":")
             if not line.isdigit():
-                raise SuggestionFailure(f'Line number must be a number. Got {line}')
+                raise SuggestionFailure(f"Line number must be a number. Got {line}")
             line_number = int(line)
             modname, node = self.find_node_by_file_and_line(file, line_number)
-            tail = node.fullname[len(modname) + 1:]  # add one to account for '.'
+            tail = node.fullname[len(modname) + 1 :]  # add one to account for '.'
         else:
             target = split_target(self.fgmanager.graph, key)
             if not target:
@@ -525,7 +565,7 @@ class SuggestionEngine:
 
         return modname, tail, node
 
-    def find_node_by_module_and_name(self, modname: str, tail: str) -> Optional[SymbolNode]:
+    def find_node_by_module_and_name(self, modname: str, tail: str) -> SymbolNode | None:
         """Find symbol node by module id and qualified name.
 
         Raise SuggestionFailure if can't find one.
@@ -538,26 +578,29 @@ class SuggestionEngine:
         names: SymbolTable = tree.names
 
         # Look through any classes
-        components = tail.split('.')
+        components = tail.split(".")
         for i, component in enumerate(components[:-1]):
             if component not in names:
-                raise SuggestionFailure("Unknown class %s.%s" %
-                                        (modname, '.'.join(components[:i + 1])))
-            node: Optional[SymbolNode] = names[component].node
+                raise SuggestionFailure(
+                    "Unknown class {}.{}".format(modname, ".".join(components[: i + 1]))
+                )
+            node: SymbolNode | None = names[component].node
             if not isinstance(node, TypeInfo):
-                raise SuggestionFailure("Object %s.%s is not a class" %
-                                        (modname, '.'.join(components[:i + 1])))
+                raise SuggestionFailure(
+                    "Object {}.{} is not a class".format(modname, ".".join(components[: i + 1]))
+                )
             names = node.names
 
         # Look for the actual function/method
         funcname = components[-1]
         if funcname not in names:
-            key = modname + '.' + tail
-            raise SuggestionFailure("Unknown %s %s" %
-                                    ("method" if len(components) > 1 else "function", key))
+            key = modname + "." + tail
+            raise SuggestionFailure(
+                "Unknown {} {}".format("method" if len(components) > 1 else "function", key)
+            )
         return names[funcname].node
 
-    def find_node_by_file_and_line(self, file: str, line: int) -> Tuple[str, SymbolNode]:
+    def find_node_by_file_and_line(self, file: str, line: int) -> tuple[str, SymbolNode]:
         """Find symbol node by path to file and line number.
 
         Find the first function declared *before or on* the line number.
@@ -565,17 +608,17 @@ class SuggestionEngine:
         Return module id and the node found. Raise SuggestionFailure if can't find one.
         """
         if not any(file.endswith(ext) for ext in PYTHON_EXTENSIONS):
-            raise SuggestionFailure('Source file is not a Python file')
+            raise SuggestionFailure("Source file is not a Python file")
         try:
             modname, _ = self.finder.crawl_up(os.path.normpath(file))
         except InvalidSourceList as e:
-            raise SuggestionFailure('Invalid source file name: ' + file) from e
+            raise SuggestionFailure("Invalid source file name: " + file) from e
         if modname not in self.graph:
-            raise SuggestionFailure('Unknown module: ' + modname)
+            raise SuggestionFailure("Unknown module: " + modname)
         # We must be sure about any edits in this file as this might affect the line numbers.
         tree = self.ensure_loaded(self.fgmanager.graph[modname], force=True)
-        node: Optional[SymbolNode] = None
-        closest_line: Optional[int] = None
+        node: SymbolNode | None = None
+        closest_line: int | None = None
         # TODO: Handle nested functions.
         for _, sym, _ in tree.local_definitions():
             if isinstance(sym.node, (FuncDef, Decorator)):
@@ -589,32 +632,35 @@ class SuggestionEngine:
                 closest_line = sym_line
                 node = sym.node
         if not node:
-            raise SuggestionFailure(f'Cannot find a function at line {line}')
+            raise SuggestionFailure(f"Cannot find a function at line {line}")
         return modname, node
 
-    def extract_from_decorator(self, node: Decorator) -> Optional[FuncDef]:
+    def extract_from_decorator(self, node: Decorator) -> FuncDef | None:
         for dec in node.decorators:
             typ = None
-            if (isinstance(dec, RefExpr)
-                    and isinstance(dec.node, FuncDef)):
+            if isinstance(dec, RefExpr) and isinstance(dec.node, FuncDef):
                 typ = dec.node.type
-            elif (isinstance(dec, CallExpr)
-                    and isinstance(dec.callee, RefExpr)
-                    and isinstance(dec.callee.node, FuncDef)
-                    and isinstance(dec.callee.node.type, CallableType)):
+            elif (
+                isinstance(dec, CallExpr)
+                and isinstance(dec.callee, RefExpr)
+                and isinstance(dec.callee.node, FuncDef)
+                and isinstance(dec.callee.node.type, CallableType)
+            ):
                 typ = get_proper_type(dec.callee.node.type.ret_type)
 
             if not isinstance(typ, FunctionLike):
                 return None
             for ct in typ.items:
-                if not (len(ct.arg_types) == 1
-                        and isinstance(ct.arg_types[0], TypeVarType)
-                        and ct.arg_types[0] == ct.ret_type):
+                if not (
+                    len(ct.arg_types) == 1
+                    and isinstance(ct.arg_types[0], TypeVarType)
+                    and ct.arg_types[0] == ct.ret_type
+                ):
                     return None
 
         return node.func
 
-    def try_type(self, func: FuncDef, typ: ProperType) -> List[str]:
+    def try_type(self, func: FuncDef, typ: ProperType) -> list[str]:
         """Recheck a function while assuming it has type typ.
 
         Return all error messages.
@@ -634,7 +680,7 @@ class SuggestionEngine:
         finally:
             func.unanalyzed_type = old
 
-    def reload(self, state: State) -> List[str]:
+    def reload(self, state: State) -> list[str]:
         """Recheck the module given by state."""
         assert state.path is not None
         self.fgmanager.flush_cache()
@@ -650,12 +696,13 @@ class SuggestionEngine:
     def named_type(self, s: str) -> Instance:
         return self.manager.semantic_analyzer.named_type(s)
 
-    def json_suggestion(self, mod: str, func_name: str, node: FuncDef,
-                        suggestion: PyAnnotateSignature) -> str:
+    def json_suggestion(
+        self, mod: str, func_name: str, node: FuncDef, suggestion: PyAnnotateSignature
+    ) -> str:
         """Produce a json blob for a suggestion suitable for application by pyannotate."""
         # pyannotate irritatingly drops class names for class and static methods
         if node.is_class or node.is_static:
-            func_name = func_name.split('.', 1)[-1]
+            func_name = func_name.split(".", 1)[-1]
 
         # pyannotate works with either paths relative to where the
         # module is rooted or with absolute paths. We produce absolute
@@ -663,35 +710,32 @@ class SuggestionEngine:
         path = os.path.abspath(self.graph[mod].xpath)
 
         obj = {
-            'signature': suggestion,
-            'line': node.line,
-            'path': path,
-            'func_name': func_name,
-            'samples': 0
+            "signature": suggestion,
+            "line": node.line,
+            "path": path,
+            "func_name": func_name,
+            "samples": 0,
         }
         return json.dumps([obj], sort_keys=True)
 
     def pyannotate_signature(
-        self,
-        cur_module: Optional[str],
-        is_method: bool,
-        typ: CallableType
+        self, cur_module: str | None, is_method: bool, typ: CallableType
     ) -> PyAnnotateSignature:
         """Format a callable type as a pyannotate dict"""
         start = int(is_method)
         return {
-            'arg_types': [self.format_type(cur_module, t) for t in typ.arg_types[start:]],
-            'return_type': self.format_type(cur_module, typ.ret_type),
+            "arg_types": [self.format_type(cur_module, t) for t in typ.arg_types[start:]],
+            "return_type": self.format_type(cur_module, typ.ret_type),
         }
 
     def format_signature(self, sig: PyAnnotateSignature) -> str:
         """Format a callable type in a way suitable as an annotation... kind of"""
         return f"({', '.join(sig['arg_types'])}) -> {sig['return_type']}"
 
-    def format_type(self, cur_module: Optional[str], typ: Type) -> str:
+    def format_type(self, cur_module: str | None, typ: Type) -> str:
         if self.use_fixme and isinstance(get_proper_type(typ), AnyType):
             return self.use_fixme
-        return typ.accept(TypeFormatter(cur_module, self.graph))
+        return typ.accept(TypeFormatter(cur_module, self.graph, self.manager.options))
 
     def score_type(self, t: Type, arg_pos: bool) -> int:
         """Generate a score for a type that we use to pick which type to use.
@@ -708,17 +752,16 @@ class SuggestionEngine:
                 return 20
             if any(has_any_type(x) for x in t.items):
                 return 15
-            if not is_optional(t):
+            if not is_overlapping_none(t):
                 return 10
         if isinstance(t, CallableType) and (has_any_type(t) or is_tricky_callable(t)):
             return 10
-        if self.try_text and isinstance(t, Instance) and t.type.fullname == 'builtins.str':
-            return 1
         return 0
 
     def score_callable(self, t: CallableType) -> int:
-        return (sum(self.score_type(x, arg_pos=True) for x in t.arg_types) +
-                self.score_type(t.ret_type, arg_pos=False))
+        return sum(self.score_type(x, arg_pos=True) for x in t.arg_types) + self.score_type(
+            t.ret_type, arg_pos=False
+        )
 
 
 def any_score_type(ut: Type, arg_pos: bool) -> float:
@@ -746,7 +789,7 @@ def any_score_type(ut: Type, arg_pos: bool) -> float:
 
 def any_score_callable(t: CallableType, is_method: bool, ignore_return: bool) -> float:
     # Ignore the first argument of methods
-    scores = [any_score_type(x, arg_pos=True) for x in t.arg_types[int(is_method):]]
+    scores = [any_score_type(x, arg_pos=True) for x in t.arg_types[int(is_method) :]]
     # Return type counts twice (since it spreads type information), unless it is
     # None in which case it does not count at all. (Though it *does* still count
     # if there are no arguments.)
@@ -763,11 +806,11 @@ def is_tricky_callable(t: CallableType) -> bool:
 
 
 class TypeFormatter(TypeStrVisitor):
-    """Visitor used to format types
-    """
+    """Visitor used to format types"""
+
     # TODO: Probably a lot
-    def __init__(self, module: Optional[str], graph: Graph) -> None:
-        super().__init__()
+    def __init__(self, module: str | None, graph: Graph, options: Options) -> None:
+        super().__init__(options=options)
         self.module = module
         self.graph = graph
 
@@ -780,7 +823,7 @@ class TypeFormatter(TypeStrVisitor):
     def visit_instance(self, t: Instance) -> str:
         s = t.type.fullname or t.type.name or None
         if s is None:
-            return '<???>'
+            return "<???>"
         if s in reverse_builtin_aliases:
             s = reverse_builtin_aliases[s]
 
@@ -792,31 +835,31 @@ class TypeFormatter(TypeStrVisitor):
         # to point to the current module. This helps the annotation tool avoid
         # inserting redundant imports when a type has been reexported.
         if self.module:
-            parts = obj.split('.')  # need to split the object part if it is a nested class
+            parts = obj.split(".")  # need to split the object part if it is a nested class
             tree = self.graph[self.module].tree
             if tree and parts[0] in tree.names:
                 mod = self.module
 
-        if (mod, obj) == ('builtins', 'tuple'):
-            mod, obj = 'typing', 'Tuple[' + t.args[0].accept(self) + ', ...]'
+        if (mod, obj) == ("builtins", "tuple"):
+            mod, obj = "typing", "Tuple[" + t.args[0].accept(self) + ", ...]"
         elif t.args:
-            obj += f'[{self.list_str(t.args)}]'
+            obj += f"[{self.list_str(t.args)}]"
 
-        if mod_obj == ('builtins', 'unicode'):
-            return 'Text'
-        elif mod == 'builtins':
+        if mod_obj == ("builtins", "unicode"):
+            return "Text"
+        elif mod == "builtins":
             return obj
         else:
-            delim = '.' if '.' not in obj else ':'
+            delim = "." if "." not in obj else ":"
             return mod + delim + obj
 
     def visit_tuple_type(self, t: TupleType) -> str:
         if t.partial_fallback and t.partial_fallback.type:
             fallback_name = t.partial_fallback.type.fullname
-            if fallback_name != 'builtins.tuple':
+            if fallback_name != "builtins.tuple":
                 return t.partial_fallback.accept(self)
         s = self.list_str(t.items)
-        return f'Tuple[{s}]'
+        return f"Tuple[{s}]"
 
     def visit_uninhabited_type(self, t: UninhabitedType) -> str:
         return "Any"
@@ -825,7 +868,7 @@ class TypeFormatter(TypeStrVisitor):
         return t.fallback.accept(self)
 
     def visit_union_type(self, t: UnionType) -> str:
-        if len(t.items) == 2 and is_optional(t):
+        if len(t.items) == 2 and is_overlapping_none(t):
             return f"Optional[{remove_optional(t).accept(self)}]"
         else:
             return super().visit_union_type(t)
@@ -845,24 +888,7 @@ class TypeFormatter(TypeStrVisitor):
         return f"Callable[{arg_str}, {t.ret_type.accept(self)}]"
 
 
-class StrToText(TypeTranslator):
-    def __init__(self, named_type: Callable[[str], Instance]) -> None:
-        self.text_type = named_type('builtins.unicode')
-
-    def visit_type_alias_type(self, t: TypeAliasType) -> Type:
-        exp_t = get_proper_type(t)
-        if isinstance(exp_t, Instance) and exp_t.type.fullname == 'builtins.str':
-            return self.text_type
-        return t.copy_modified(args=[a.accept(self) for a in t.args])
-
-    def visit_instance(self, t: Instance) -> Type:
-        if t.type.fullname == 'builtins.str':
-            return self.text_type
-        else:
-            return super().visit_instance(t)
-
-
-TType = TypeVar('TType', bound=Type)
+TType = TypeVar("TType", bound=Type)
 
 
 def make_suggestion_anys(t: TType) -> TType:
@@ -885,7 +911,7 @@ class MakeSuggestionAny(TypeTranslator):
         return t.copy_modified(args=[a.accept(self) for a in t.args])
 
 
-def generate_type_combinations(types: List[Type]) -> List[Type]:
+def generate_type_combinations(types: list[Type]) -> list[Type]:
     """Generate possible combinations of a list of types.
 
     mypy essentially supports two different ways to do this: joining the types
@@ -893,14 +919,14 @@ def generate_type_combinations(types: List[Type]) -> List[Type]:
     """
     joined_type = join_type_list(types)
     union_type = make_simplified_union(types)
-    if is_same_type(joined_type, union_type):
+    if joined_type == union_type:
         return [joined_type]
     else:
         return [joined_type, union_type]
 
 
-def count_errors(msgs: List[str]) -> int:
-    return len([x for x in msgs if ' error: ' in x])
+def count_errors(msgs: list[str]) -> int:
+    return len([x for x in msgs if " error: " in x])
 
 
 def refine_type(ti: Type, si: Type) -> Type:
@@ -1009,11 +1035,11 @@ def refine_callable(t: CallableType, s: CallableType) -> CallableType:
     )
 
 
-T = TypeVar('T')
+T = TypeVar("T")
 
 
-def dedup(old: List[T]) -> List[T]:
-    new: List[T] = []
+def dedup(old: list[T]) -> list[T]:
+    new: list[T] = []
     for x in old:
         if x not in new:
             new.append(x)
