@@ -63,9 +63,9 @@ def test_python_evaluation(testcase: DataDrivenTestCase, cache_dir: str) -> None
         additional_flags = m.group(1).split()
         for flag in additional_flags:
             if flag.startswith("--python-version="):
-                targetted_python_version = flag.split("=")[1]
-                targetted_major, targetted_minor = targetted_python_version.split(".")
-                if (int(targetted_major), int(targetted_minor)) > (
+                targeted_python_version = flag.split("=")[1]
+                targeted_major, targeted_minor = targeted_python_version.split(".")
+                if (int(targeted_major), int(targeted_minor)) > (
                     sys.version_info.major,
                     sys.version_info.minor,
                 ):
